
#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <cmath>
#include <ctime>

class Screen {
    int w,h;
    char* arr;
public:
    // Screen(23,23);
    Screen(int w,int h){
        this->w = w;
        this->h = h;
        arr = new char[w * h];
    }
    // Screen a = b;    <--- copy constructor.
    Screen(const Screen& s){
        w = s.w;
        h = s.h;
        arr = new char[w * h];
        memcpy(arr,s.arr,w*h*sizeof(char));
    }
    // Screen a,b;
    ///  a = b;  <-- this is whre this applies.
    ///// a.operator=(b)  <-- semanitically 
    Screen& operator=(const Screen& s){
        w = s.w;
        h = s.h;
        arr = new char[w * h];
        memcpy(arr,s.arr,w*h*sizeof(char));        
    }
    ~Screen(){
        delete[] arr;
        arr = 0;
        w = h = 0;
    }
    
    int getWidth(){
        return w;
    }
    int getHeight(){
        return h;
    }

    // put character c in location x,y;
    void plot(int x,int y,char c){
        x = x % w;  // sanity check.
        y = y % h;
        arr[ y * w + x ] = c;
    }

    // draw a circle at x,y, with radius r, using chars c.
    void circle(int x,int y,int r,char c){
        float pi = atan(1)*4;
        for(float theta = 0; theta <= pi*2; theta += pi/360){
            float xt = x + cos(theta) * r;
            float yt = y + sin(theta) * r;
            plot((int)xt,(int)yt,c);
        }
    }

    // draw a line from x1,y1, to x2,y2 using c.
    void line(int x1,int y1,int x2,int y2,char c){
        //printf("(%d,%d) => (%d,%d)\n",x1,y1,x2,y2);
        if(abs(x1-x2)<=1 && abs(y1-y2)<=1){
            plot(x1,y1,c);
            return;
        }
        int cx = (x2+x1)/2;
        int cy = (y2+y1)/2;
        line(x1,y1,cx,cy,c);
        line(cx,cy,x2,y2,c);
    }

    // get character at location x,y
    char getat(int x,int y){
        x = x % w;  // sanity check.
        y = y % h;
        return arr[ y * w + x ] > ' ' ? arr[ y * w + x ] : ' ';
    }


    // dump our screen onto the "real" screen.
    void blit(){
        // draw top border.
        fputc('|',stdout);            
        for(int j=0;j<w;j++)
            fputc('-',stdout);
        fputc('|',stdout);            
        fputc('\n',stdout);
        
        // draw content.
        for(int i=0;i<h;i++){
            fputc('|',stdout);  // draw left side border.
            for(int j=0;j<w;j++){
                fputc(getat(j,i) ,stdout);  // draw content.
            }
            fputc('|',stdout);  // draw right side border.
            fputc('\n',stdout);
        }

        // draw bottom border.
        fputc('|',stdout);            
        for(int j=0;j<w;j++)
            fputc('-',stdout);
        fputc('|',stdout);            
        fputc('\n',stdout);        
    }
};


int main(){
    Screen s(50,20);

    //s.plot(20,10,'#');
    //s.circle(20,10,3,'@');
    //s.circle(20,7,7,'.');

    //s.line(3,3,10,10,'*');
    //s.blit();

    time_t tim;
    time(&tim);
    tm* t = localtime(&tim);
    printf("%d:%d\n",t->tm_hour,t->tm_min);

    s.circle(s.getWidth()/2,s.getHeight()/2,s.getHeight()/2 - 1,'.');

    int h = t->tm_hour % 12; 

    float pi = atan(1)*4;
    {
        float theta = (2.0*pi / 12.0 * h) - pi/2;
        float r =  (s.getHeight()/2 - 3);
        float hx = s.getWidth()/2 + cos(theta) * r;
        float hy = s.getHeight()/2 + sin(theta) * r;
        s.line(s.getWidth()/2,s.getHeight()/2,hx,hy,'*');
    }
    {
        float theta = (2.0*pi / 60.0 * t->tm_min) - pi/2;
        float r =  (s.getHeight()/2 - 1);
        float hx = s.getWidth()/2 + cos(theta) * r;
        float hy = s.getHeight()/2 + sin(theta) * r;
        s.line(s.getWidth()/2,s.getHeight()/2,hx,hy,'+');
    }
    
    s.blit();
}


